<?php /* ==========================================================================
                                                                                  |
    Скрипт: Хелпер шаблона.                                                       |
    ----------------------------------------------------------------------------  |
                                                                                  |
    Служит цели снабдить шаблон какими-либо дополнительными шаблонизационными     |
    функциями или переопределить дефолтные.                                       |
                                                                                  |
================================================================================ */

    class TemplateEmulator {
        protected $cms = null;

        /* ========================================================================
        |                                                                         |
        |    Регистрация наших шаблонизационных функций.                          |
        |                                                                         |
        ======================================================================== */

        protected function setSmartyPlugins ( & $cms ) {
            $items = array('version',              // вывести номер версии движка
                           'versionYMD',           // вывести ДАТА-номер версии
                           'header200',            // отправить серверный заголовок 200 OK
                           'metaDescription',      // вывести мета описание страницы
                           'metaKeywords',         // вывести мета ключевые слова страницы
                           'siteName',             // вывести название сайта
                           'phone',                // вывести телефон, указанный в конфиге
                           'email',                // вывести емейл, указанный в конфиге
                           'year',                 // вывести текущий год
                           'counters',             // вывести коды счетчиков
                           'pagination',           // вывести html-контент пагинации страниц
                           'date',                 // вывести дату записи
                           'id');                  // вывести идентификатор записи
            foreach ($items as $name) {
                $cms->smarty->registerPlugin('function', $name, array($this, $name));
            }
            $cms->smarty->assignByRef('emulator', $this);
        }

        /* ========================================================================
        |                                                                         |
        |    Обработчики наших функций.                                           |
        |                                                                         |
        ======================================================================== */

        public function version ( $params = null, & $smarty = null ) {
            $value = $this->getConst('IMPERA_CMS_CURRENT_MMVERSION');
            return $this->asAttribute($value);
        }

        public function versionYMD ( $params = null, & $smarty = null ) {
            $value = $this->getConst('IMPERA_CMS_CURRENT_VERSION');
            return $this->asAttribute($value);
        }

        public function header200 ( $params = null, & $smarty = null ) {
            @ header('HTTP/1.1 200 OK', TRUE, 200);
            return '';
        }

        public function metaDescription ( $params = null, & $smarty = null ) {
            $value = $this->getVar('description', $smarty);
            $value = $this->mustbeString($value);
            return $this->asAttribute($value);
        }

        public function metaKeywords ( $params = null, & $smarty = null ) {
            $value = $this->getVar('keywords', $smarty);
            $value = $this->mustbeString($value);
            return $this->asAttribute($value);
        }

        public function siteName ( $params = null, & $smarty = null ) {
            $value = $this->cms->any->settings->getAsSentence('site_name');
            return $this->asAttribute($value);
        }

        public function phone ( $params = null, & $smarty = null ) {
            $value = $this->getConfig('phone1');
            return $this->asAttribute($value);
        }

        public function email ( $params = null, & $smarty = null ) {
            $value = $this->getConfig('email1');
            return $this->asAttribute($value);
        }

        public function year ( $params = null, & $smarty = null ) {
            return date('Y', time());
        }

        public function counters ( $params = null, & $smarty = null ) {
            $value = $this->cms->any->settings->get('counters');
            return $this->mustbeString($value);
        }

        public function pagination ( $params = null, & $smarty = null ) {
            $value = $this->getVar('PagesNavigation', $smarty);
            return $this->mustbeString($value);
        }

        public function date ( $params = null, & $smarty = null ) {
            $item = $this->getParam('item', $params);
            if (is_null($item)) $item = $this->getVar('item', $smarty);
            if (!empty($item->date_date)) $item = $item->date_date;
            elseif (!empty($item->date)) $item = $item->date;
            elseif (!empty($item->created)) $item = $item->created;
            elseif (!is_string($item)) $item = '1970-01-01 00:00';
            $names = $this->getMonths();
            $value = $this->getDay($item) . ' ' . $names[$this->getMonth($item) - 1] . ' ' . $this->getYear($item);
            if ($this->getParam('time', $params)) $value .= ' в ' . $this->getHour($item, TRUE) . ':' . $this->getMinute($item, TRUE);
            return $value;
        }

        public function id ( $params = null, & $smarty = null ) {
            $varname = $this->getParam('from', $params, 'item');
            $item = $this->getVar($varname, $smarty);
            $fields = array('comment_id',
                            'section_id',
                            'variant_id', 'product_id', 'kit_id', 'property_id',
                            'article_id', 'news_id',
                            'file_id',
                            'stock_id',
                            'brand_id',
                            'event_id',
                            'country_id', 'region_id', 'town_id',
                            'category_id',
                            'id',
                            'menu_id',
                            'order_id',
                            'user_id');
            foreach ($fields as $field) if (isset($item->$field)) return $this->asAttribute($item->$field);
            return 0;
        }

        /* ========================================================================
        |                                                                         |
        |    Функции объекта $emulator.                                           |
        |                                                                         |
        ======================================================================== */

        public function checkNoPage () {
            return $this->getConst('CONTENT_MESSAGE_NO_PAGE') === $this->getVar('content');
        }

        public function checkNoModule () {
            $v = $this->getConst('CONTENT_MESSAGE_NO_MODULE');
                $v = preg_replace('/([~#:\+\[\]\{\}\(\)\?\.\$\^\|\\\\])/u', '\\\\$1', $v);
                $v = preg_replace('/\*/u', '[a-z0-9_]+', $v);
            return preg_match('~^' . $v . '$~iu', $this->getVar('content')) > 0;
        }

        public function existsModule ( $file ) {
            $file = $this->cms->hdd->safeFilename($file);
            if (empty($file)) return FALSE;
            $ext = explode('.', $file);
            if (count($ext) < 2) return FALSE;
            $ext = array_pop($ext);
            $ext = $this->cms->text->lowerCase($ext);
            if (!in_array($ext, array('tpl', 'htm', 'html', 'xml', 'css', 'js', 'txt'))) return FALSE;
            return $this->cms->hdd->isReadableFile(dirname(__FILE__) . '/' . $file);
        }

        public function existsUser ( $login = '' ) {
            if ($this->getUserId() == 0) return FALSE;
            return empty($login)
                   || isset($this->cms->user->email) && $this->cms->user->email == $login
                   || isset($this->cms->user->email2) && $this->cms->user->email2 == $login;
        }

        public function existsAdmin ( $login = '' ) {
            if (!$this->cms->security->existsAdmin()) return FALSE;
            return empty($login)
                   || $login == $this->cms->security->currentAdminLogin();
        }

        public function accessibleOrder ( $item = null ) {
            if (is_null($item)) $item = $this->getVar('order');
            if (empty($item->order_id)) return FALSE;
            return empty($item->hidden)
                   || $this->existsAdmin()
                   || $this->existsUser() && $this->getUserId() == $this->getUserId($item);
        }

        /* ========================================================================
        |                                                                         |
        |    Системные методы.                                                    |
        |                                                                         |
        ======================================================================== */

        public function __construct ( & $cms ) {
            $this->cms = & $cms;
            $this->setSmartyPlugins($cms);
        }

        private function mustbeString ( & $value, $def = '' ) {
            return is_string($value) ? $value : $def;
        }

        private function asAttribute ( & $value ) {
            return htmlspecialchars($value, ENT_QUOTES, 'UTF-8');
        }

        private function getParam ( $name, & $params, $def = null ) {
            return isset($params[$name]) ? $params[$name] : $def;
        }

        private function getConst ( $name, $def = '' ) {
            return defined($name) ? constant($name) : $def;
        }

        private function getConfig ( $name, $def = '' ) {
            return isset($this->cms->config->$name) ? trim($this->cms->config->$name) : $def;
        }

        private function getVar ( $name, & $smarty = null, $def = null ) {
            $value = null;
            if (is_string($name) && !empty($name)) {
                if (!is_null($smarty)) $value = $smarty->getTemplateVars($name);
                if (is_null($value)) $value = $this->cms->smarty->getTemplateVars($name);
            }
            return is_null($value) ? $def : $value;
        }

        private function getUserId ( $item = null ) {
            if (is_object($item)) return isset($item->user_id) ? intval($item->user_id) : 0;
            return isset($this->cms->user->user_id) ? intval($this->cms->user->user_id) : 0;
        }

        private function getMonths ( $short = TRUE ) {
            $items = array('января', 'февраля', 'марта', 'апреля', 'мая', 'июня', 'июля', 'августа', 'сентября', 'октября', 'ноября', 'декабря');
            if ($short) foreach ($items as & $item) $item = $this->cms->text->substr($item, 0, 3);
            return $items;
        }

        private function getDay ( $yyyy_mm_dd, $forClock = FALSE ) {
            $offset = preg_match('/^[0-9][0-9][^0-9]/', $yyyy_mm_dd) ? 0 : 8;
            $value = intval(max(1, min(substr($yyyy_mm_dd, $offset, 2), 31)));
            return $forClock ? sprintf('%02d', $value) : $value;
        }

        private function getMonth ( $yyyy_mm_dd, $forClock = FALSE ) {
            $offset = preg_match('/^[0-9][0-9][^0-9]/', $yyyy_mm_dd) ? 3 : 5;
            $value = intval(max(1, min(substr($yyyy_mm_dd, $offset, 2), 12)));
            return $forClock ? sprintf('%02d', $value) : $value;
        }

        private function getYear ( $yyyy_mm_dd ) {
            $offset = preg_match('/^[0-9][0-9][^0-9]/', $yyyy_mm_dd) ? 6 : 0;
            $now = date('Y');
            return intval(max(1970, min(substr($yyyy_mm_dd, $offset, 4), $now)));
        }

        private function getHour ( $yyyy_mm_dd_hh_mm, $forClock = FALSE ) {
            $value = intval(max(0, min(substr($yyyy_mm_dd_hh_mm, 11, 2), 23)));
            return $forClock ? sprintf('%02d', $value) : $value;
        }

        private function getMinute ( $yyyy_mm_dd_hh_mm, $forClock = FALSE ) {
            $value = intval(max(0, min(substr($yyyy_mm_dd_hh_mm, 14, 2), 59)));
            return $forClock ? sprintf('%02d', $value) : $value;
        }
    }

    return;
?>